@@ -190,6 +190,7 @@ class IOPubThread:
             event_pipe.close()
 
     def close(self):
+        """Close the IOPub thread."""
         if self.closed:
             return
         self.socket.close()
@@ -244,6 +245,7 @@ class BackgroundSocket:
     io_thread = None
 
     def __init__(self, io_thread):
+        """Initialize the socket."""
         self.io_thread = io_thread
 
     def __getattr__(self, attr):
@@ -264,7 +266,8 @@ class BackgroundSocket:
         super().__getattr__(attr)  # type:ignore[misc]
 
     def __setattr__(self, attr, value):
-        if attr == "io_thread" or (attr.startswith("__" and attr.endswith("__"))):
+        """Set an attribute on the socket."""
+        if attr == "io_thread" or (attr.startswith("__") and attr.endswith("__")):
             super().__setattr__(attr, value)
         else:
             warnings.warn(
@@ -278,6 +281,7 @@ class BackgroundSocket:
             setattr(self.io_thread.socket, attr, value)
 
     def send(self, msg, *args, **kwargs):
+        """Send a message to the socket."""
         return self.send_multipart([msg], *args, **kwargs)
 
     def send_multipart(self, *args, **kwargs):
@@ -308,7 +312,8 @@ class OutStream(TextIOBase):
         if getattr(self, "_original_stdstream_copy", None) is not None:
             return self._original_stdstream_copy
         else:
-            raise io.UnsupportedOperation("fileno")
+            msg = "fileno"
+            raise io.UnsupportedOperation(msg)
 
     def _watch_pipe_fd(self):
         """
@@ -411,7 +416,8 @@ class OutStream(TextIOBase):
             if hasattr(echo, "read") and hasattr(echo, "write"):
                 self.echo = echo
             else:
-                raise ValueError("echo argument must be a file like object")
+                msg = "echo argument must be a file like object"
+                raise ValueError(msg)
 
     def isatty(self):
         """Return a bool indicating whether this is an 'interactive' stream.
@@ -438,9 +444,11 @@ class OutStream(TextIOBase):
         return os.getpid() == self._master_pid
 
     def set_parent(self, parent):
+        """Set the parent header."""
         self.parent_header = extract_header(parent)
 
     def close(self):
+        """Close the stream."""
         if sys.platform.startswith("linux") or sys.platform.startswith("darwin"):
             self._should_watch = False
             self.watch_fd_thread.join()
@@ -534,7 +542,8 @@ class OutStream(TextIOBase):
         """
 
         if not isinstance(string, str):
-            raise TypeError(f"write() argument must be str, not {type(string)}")
+            msg = f"write() argument must be str, not {type(string)}"
+            raise TypeError(msg)
 
         if self.echo is not None:
             try:
@@ -544,7 +553,8 @@ class OutStream(TextIOBase):
                     print(f"Write failed: {e}", file=sys.__stderr__)
 
         if self.pub_thread is None:
-            raise ValueError("I/O operation on closed file")
+            msg = "I/O operation on closed file"
+            raise ValueError(msg)
         else:
 
             is_child = not self._is_master_process()
@@ -565,13 +575,16 @@ class OutStream(TextIOBase):
         return len(string)
 
     def writelines(self, sequence):
+        """Write lines to the stream."""
         if self.pub_thread is None:
-            raise ValueError("I/O operation on closed file")
+            msg = "I/O operation on closed file"
+            raise ValueError(msg)
         else:
             for string in sequence:
                 self.write(string)
 
     def writable(self):
+        """Test whether the stream is writable."""
         return True
 
     def _flush_buffer(self):
