@@ -8,7 +8,7 @@ import numbers
 import re
 import types
 from binascii import b2a_base64
-from datetime import datetime
+from datetime import date, datetime
 
 from jupyter_client._version import version_info as jupyter_client_version
 
@@ -74,7 +74,7 @@ def encode_images(format_dict):
     return format_dict
 
 
-def json_clean(obj):
+def json_clean(obj):  # pragma: no cover
     """Deprecated, this is a no-op for jupyter-client>=7.
 
     Clean an object to ensure it's safe to encode in JSON.
@@ -146,16 +146,17 @@ def json_clean(obj):
         nkeys = len(obj)
         nkeys_collapsed = len(set(map(str, obj)))
         if nkeys != nkeys_collapsed:
-            raise ValueError(
+            msg = (
                 "dict cannot be safely converted to JSON: "
                 "key collision would lead to dropped values"
             )
+            raise ValueError(msg)
         # If all OK, proceed by making the new dict that will be json-safe
         out = {}
         for k, v in obj.items():
             out[str(k)] = json_clean(v)
         return out
-    if isinstance(obj, datetime):
+    if isinstance(obj, datetime) or isinstance(obj, date):
         return obj.strftime(ISO8601)
 
     # we don't understand it, it's probably an unserializable object
