@@ -15,7 +15,7 @@ from io import FileIO, TextIOWrapper
 from logging import StreamHandler
 
 import zmq
-from IPython.core.application import (
+from IPython.core.application import (  # type:ignore[attr-defined]
     BaseIPythonApplication,
     base_aliases,
     base_flags,
@@ -88,12 +88,12 @@ kernel_flags.update(
 )
 
 # inherit flags&aliases for any IPython shell apps
-kernel_aliases.update(shell_aliases)
+kernel_aliases.update(shell_aliases)  # type:ignore[arg-type]
 kernel_flags.update(shell_flags)
 
 # inherit flags&aliases for Sessions
-kernel_aliases.update(session_aliases)
-kernel_flags.update(session_flags)
+kernel_aliases.update(session_aliases)  # type:ignore[arg-type]
+kernel_flags.update(session_flags)  # type:ignore[arg-type]
 
 _ctrl_c_message = """\
 NOTE: When using the `ipython kernel` entry point, Ctrl-C will not work.
@@ -111,9 +111,11 @@ To read more about this, see https://github.com/ipython/ipython/issues/2049
 
 
 class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMixin):
+    """The IPYKernel application class."""
+
     name = "ipython-kernel"
-    aliases = Dict(kernel_aliases)
-    flags = Dict(kernel_flags)
+    aliases = Dict(kernel_aliases)  # type:ignore[assignment]
+    flags = Dict(kernel_flags)  # type:ignore[assignment]
     classes = [IPythonKernel, ZMQInteractiveShell, ProfileDir, Session]
     # the kernel class, as an importstring
     kernel_class = Type(
@@ -168,7 +170,9 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     trio_loop = Bool(False, help="Set main event loop.").tag(config=True)
     quiet = Bool(True, help="Only send stdout/stderr to output stream").tag(config=True)
     outstream_class = DottedObjectName(
-        "ipykernel.iostream.OutStream", help="The importstring for the OutStream factory"
+        "ipykernel.iostream.OutStream",
+        help="The importstring for the OutStream factory",
+        allow_none=True,
     ).tag(config=True)
     displayhook_class = DottedObjectName(
         "ipykernel.displayhook.ZMQDisplayHook", help="The importstring for the DisplayHook factory"
@@ -195,13 +199,16 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
     ).tag(config=True)
 
     def init_crash_handler(self):
+        """Initialize the crash handler."""
         sys.excepthook = self.excepthook
 
     def excepthook(self, etype, evalue, tb):
+        """Handle an exception."""
         # write uncaught traceback to 'real' stderr, not zmq-forwarder
         traceback.print_exception(etype, evalue, tb, file=sys.__stderr__)
 
     def init_poller(self):
+        """Initialize the poller."""
         if sys.platform == "win32":
             if self.interrupt or self.parent_handle:
                 self.poller = ParentPollerWindows(self.interrupt, self.parent_handle)
@@ -266,6 +273,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         )
 
     def cleanup_connection_file(self):
+        """Clean up our connection file."""
         cf = self.abs_connection_file
         self.log.debug("Cleaning up connection file: %s", cf)
         try:
@@ -276,6 +284,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         self.cleanup_ipc_files()
 
     def init_connection_file(self):
+        """Initialize our connection file."""
         if not self.connection_file:
             self.connection_file = "kernel-%s.json" % os.getpid()
         try:
@@ -296,7 +305,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             self.exit(1)
 
     def init_sockets(self):
-        # Create a context, a session, and the kernel sockets.
+        """Create a context, a session, and the kernel sockets."""
         self.log.info("Starting the kernel at pid: %i", os.getpid())
         assert self.context is None, "init_sockets cannot be called twice!"
         self.context = context = zmq.Context()
@@ -322,6 +331,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         self.init_iopub(context)
 
     def init_control(self, context):
+        """Initialize the control channel."""
         self.control_socket = context.socket(zmq.ROUTER)
         self.control_socket.linger = 1000
         self.control_port = self._bind_socket(self.control_socket, self.control_port)
@@ -344,6 +354,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         self.control_thread = ControlThread(daemon=True)
 
     def init_iopub(self, context):
+        """Initialize the iopub channel."""
         self.iopub_socket = context.socket(zmq.PUB)
         self.iopub_socket.linger = 1000
         self.iopub_port = self._bind_socket(self.iopub_socket, self.iopub_port)
@@ -418,7 +429,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             self.log.info(line)
         # also raw print to the terminal if no parent_handle (`ipython kernel`)
         # unless log-level is CRITICAL (--quiet)
-        if not self.parent_handle and self.log_level < logging.CRITICAL:
+        if not self.parent_handle and int(self.log_level) < logging.CRITICAL:
             print(_ctrl_c_message, file=sys.__stdout__)
             for line in lines:
                 print(line, file=sys.__stdout__)
@@ -515,6 +526,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
                 faulthandler.register = register
 
     def init_signal(self):
+        """Initialize the signal handler."""
         signal.signal(signal.SIGINT, signal.SIG_IGN)
 
     def init_kernel(self):
@@ -578,6 +590,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             shell._showtraceback = _showtraceback
 
     def init_shell(self):
+        """Initialize the shell channel."""
         self.shell = getattr(self.kernel, "shell", None)
         if self.shell:
             self.shell.configurables.append(self)
@@ -623,10 +636,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
             import asyncio
 
             try:
-                from asyncio import (
-                    WindowsProactorEventLoopPolicy,
-                    WindowsSelectorEventLoopPolicy,
-                )
+                from asyncio import WindowsProactorEventLoopPolicy, WindowsSelectorEventLoopPolicy
             except ImportError:
                 pass
                 # not affected
@@ -642,18 +652,19 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         With the non-interruptible version, stopping pdb() locks up the kernel in a
         non-recoverable state.
         """
-        import pdb
+        import pdb  # noqa
 
         from IPython.core import debugger
 
         if hasattr(debugger, "InterruptiblePdb"):
             # Only available in newer IPython releases:
-            debugger.Pdb = debugger.InterruptiblePdb
-            pdb.Pdb = debugger.Pdb  # type:ignore[misc]
-            pdb.set_trace = debugger.set_trace
+            debugger.Pdb = debugger.InterruptiblePdb  # type:ignore
+            pdb.Pdb = debugger.Pdb  # type:ignore
+            pdb.set_trace = debugger.set_trace  # type:ignore[assignment]
 
     @catch_config_error
     def initialize(self, argv=None):
+        """Initialize the application."""
         self._init_asyncio_patch()
         super().initialize(argv)
         if self.subapp is not None:
@@ -676,7 +687,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         except Exception:
             # Catch exception when initializing signal fails, eg when running the
             # kernel on a separate thread
-            if self.log_level < logging.CRITICAL:
+            if int(self.log_level) < logging.CRITICAL:
                 self.log.error("Unable to initialize signal:", exc_info=True)
         self.init_kernel()
         # shell init steps
@@ -692,6 +703,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
         sys.stderr.flush()
 
     def start(self):
+        """Start the application."""
         if self.subapp is not None:
             return self.subapp.start()
         if self.poller is not None:
@@ -717,7 +729,7 @@ class IPKernelApp(BaseIPythonApplication, InteractiveShellApp, ConnectionFileMix
 launch_new_instance = IPKernelApp.launch_instance
 
 
-def main():
+def main():  # pragma: no cover
     """Run an IPKernel as an application"""
     app = IPKernelApp.instance()
     app.initialize()
