@@ -68,6 +68,7 @@ def _accepts_cell_id(meth):
 
 
 class Kernel(SingletonConfigurable):
+    """The base kernel class."""
 
     # ---------------------------------------------------------------------------
     # Kernel interface
@@ -102,7 +103,7 @@ class Kernel(SingletonConfigurable):
     banner: str
 
     @default("shell_streams")
-    def _shell_streams_default(self):
+    def _shell_streams_default(self):  # pragma: no cover
         warnings.warn(
             "Kernel.shell_streams is deprecated in ipykernel 6.0. Use Kernel.shell_stream",
             DeprecationWarning,
@@ -114,7 +115,7 @@ class Kernel(SingletonConfigurable):
             return []
 
     @observe("shell_streams")
-    def _shell_streams_changed(self, change):
+    def _shell_streams_changed(self, change):  # pragma: no cover
         warnings.warn(
             "Kernel.shell_streams is deprecated in ipykernel 6.0. Use Kernel.shell_stream",
             DeprecationWarning,
@@ -259,6 +260,7 @@ class Kernel(SingletonConfigurable):
     ]
 
     def __init__(self, **kwargs):
+        """Initialize the kernel."""
         super().__init__(**kwargs)
         # Build dict of handlers for message types
         self.shell_handlers = {}
@@ -683,7 +685,6 @@ class Kernel(SingletonConfigurable):
 
     async def execute_request(self, stream, ident, parent):
         """handle an execute_request"""
-
         try:
             content = parent["content"]
             code = content["code"]
@@ -770,6 +771,7 @@ class Kernel(SingletonConfigurable):
         raise NotImplementedError
 
     async def complete_request(self, stream, ident, parent):
+        """Handle a completion request."""
         content = parent["content"]
         code = content["code"]
         cursor_pos = content["cursor_pos"]
@@ -792,6 +794,7 @@ class Kernel(SingletonConfigurable):
         }
 
     async def inspect_request(self, stream, ident, parent):
+        """Handle an inspect request."""
         content = parent["content"]
 
         reply_content = self.do_inspect(
@@ -813,6 +816,7 @@ class Kernel(SingletonConfigurable):
         return {"status": "ok", "data": {}, "metadata": {}, "found": False}
 
     async def history_request(self, stream, ident, parent):
+        """Handle a history request."""
         content = parent["content"]
 
         reply_content = self.do_history(**content)
@@ -839,6 +843,7 @@ class Kernel(SingletonConfigurable):
         return {"status": "ok", "history": []}
 
     async def connect_request(self, stream, ident, parent):
+        """Handle a connect request."""
         if self._recorded_ports is not None:
             content = self._recorded_ports.copy()
         else:
@@ -859,12 +864,14 @@ class Kernel(SingletonConfigurable):
         }
 
     async def kernel_info_request(self, stream, ident, parent):
+        """Handle a kernel info request."""
         content = {"status": "ok"}
         content.update(self.kernel_info)
         msg = self.session.send(stream, "kernel_info_reply", content, parent, ident)
         self.log.debug("%s", msg)
 
     async def comm_info_request(self, stream, ident, parent):
+        """Handle a comm info request."""
         content = parent["content"]
         target_name = content.get("target_name", None)
 
@@ -872,7 +879,7 @@ class Kernel(SingletonConfigurable):
         if hasattr(self, "comm_manager"):
             comms = {
                 k: dict(target_name=v.target_name)
-                for (k, v) in self.comm_manager.comms.items()  # type:ignore[attr-defined]
+                for (k, v) in self.comm_manager.comms.items()
                 if v.target_name == target_name or target_name is None
             }
         else:
@@ -901,12 +908,14 @@ class Kernel(SingletonConfigurable):
                 pass
 
     async def interrupt_request(self, stream, ident, parent):
+        """Handle an interrupt request."""
         self._send_interupt_children()
         content = parent["content"]
         self.session.send(stream, "interrupt_reply", content, parent, ident=ident)
         return
 
     async def shutdown_request(self, stream, ident, parent):
+        """Handle a shutdown request."""
         content = self.do_shutdown(parent["content"]["restart"])
         if inspect.isawaitable(content):
             content = await content
@@ -931,6 +940,7 @@ class Kernel(SingletonConfigurable):
         return {"status": "ok", "restart": restart}
 
     async def is_complete_request(self, stream, ident, parent):
+        """Handle an is_complete request."""
         content = parent["content"]
         code = content["code"]
 
@@ -946,8 +956,8 @@ class Kernel(SingletonConfigurable):
         return {"status": "unknown"}
 
     async def debug_request(self, stream, ident, parent):
+        """Handle a debug request."""
         content = parent["content"]
-
         reply_content = self.do_debug_request(content)
         if inspect.isawaitable(reply_content):
             reply_content = await reply_content
@@ -956,6 +966,7 @@ class Kernel(SingletonConfigurable):
         self.log.debug("%s", reply_msg)
 
     def get_process_metric_value(self, process, name, attribute=None):
+        """Get the process metric value."""
         try:
             metric_value = getattr(process, name)()
             if attribute is not None:  # ... a named tuple
@@ -968,6 +979,7 @@ class Kernel(SingletonConfigurable):
             return None
 
     async def usage_request(self, stream, ident, parent):
+        """Handle a usage request."""
         reply_content = {"hostname": socket.gethostname(), "pid": os.getpid()}
         current_process = psutil.Process()
         all_processes = [current_process] + current_process.children(recursive=True)
@@ -1006,7 +1018,8 @@ class Kernel(SingletonConfigurable):
     # Engine methods (DEPRECATED)
     # ---------------------------------------------------------------------------
 
-    async def apply_request(self, stream, ident, parent):
+    async def apply_request(self, stream, ident, parent):  # pragma: no cover
+        """Handle an apply request."""
         self.log.warning("apply_request is deprecated in kernel_base, moving to ipyparallel.")
         try:
             content = parent["content"]
@@ -1044,7 +1057,7 @@ class Kernel(SingletonConfigurable):
     # Control messages (DEPRECATED)
     # ---------------------------------------------------------------------------
 
-    async def abort_request(self, stream, ident, parent):
+    async def abort_request(self, stream, ident, parent):  # pragma: no cover
         """abort a specific msg by id"""
         self.log.warning(
             "abort_request is deprecated in kernel_base. It is only part of IPython parallel"
@@ -1063,7 +1076,7 @@ class Kernel(SingletonConfigurable):
         )
         self.log.debug("%s", reply_msg)
 
-    async def clear_request(self, stream, idents, parent):
+    async def clear_request(self, stream, idents, parent):  # pragma: no cover
         """Clear our namespace."""
         self.log.warning(
             "clear_request is deprecated in kernel_base. It is only part of IPython parallel"
@@ -1133,9 +1146,8 @@ class Kernel(SingletonConfigurable):
     def _no_raw_input(self):
         """Raise StdinNotImplementedError if active frontend doesn't support
         stdin."""
-        raise StdinNotImplementedError(
-            "raw_input was called, but this frontend does not support stdin."
-        )
+        msg = "raw_input was called, but this frontend does not support stdin."
+        raise StdinNotImplementedError(msg)
 
     def getpass(self, prompt="", stream=None):
         """Forward getpass to frontends
@@ -1145,9 +1157,8 @@ class Kernel(SingletonConfigurable):
         StdinNotImplementedError if active frontend doesn't support stdin.
         """
         if not self._allow_stdin:
-            raise StdinNotImplementedError(
-                "getpass was called, but this frontend does not support input requests."
-            )
+            msg = "getpass was called, but this frontend does not support input requests."
+            raise StdinNotImplementedError(msg)
         if stream is not None:
             import warnings
 
@@ -1171,9 +1182,8 @@ class Kernel(SingletonConfigurable):
         StdinNotImplementedError if active frontend doesn't support stdin.
         """
         if not self._allow_stdin:
-            raise StdinNotImplementedError(
-                "raw_input was called, but this frontend does not support input requests."
-            )
+            msg = "raw_input was called, but this frontend does not support input requests."
+            raise StdinNotImplementedError(msg)
         return self._input_request(
             str(prompt),
             self._parent_ident["shell"],
@@ -1216,7 +1226,8 @@ class Kernel(SingletonConfigurable):
                         break
             except KeyboardInterrupt:
                 # re-raise KeyboardInterrupt, to truncate traceback
-                raise KeyboardInterrupt("Interrupted by user") from None
+                msg = "Interrupted by user"
+                raise KeyboardInterrupt(msg) from None
             except Exception:
                 self.log.warning("Invalid Message:", exc_info=True)
 
